-- displays data gathered by the core in FuBar
local name = ...
local WhoRoll = WhoRoll
local debug = WhoRoll.debug

local L = LibStub("AceLocale-3.0"):GetLocale(name)

local qt = LibStub('LibQTip-1.0')
local TOOLTIPNAME = "WhoRollTooltip"

-- globals
local _G = _G
local NEED = _G.NEED
local GREED = _G.GREED
local PASS = _G.PASS
local DISENCHANT = "DISENCHANT"
local GetItemInfo = _G.GetItemInfo
local GetItemQualityColor = _G.GetItemQualityColor
local UnitInRaid = _G.UnitInRaid
local UnitInParty = _G.UnitInParty
local GetNumPartyMembers = _G.GetNumPartyMembers
local GetNumRaidMembers = _G.GetNumRaidMembers
local UnitName = _G.UnitName

local lootIcons = {}
lootIcons[LOOT_ROLL_TYPE_NEED]= "Interface/Buttons/UI-GroupLoot-Dice-Up"
lootIcons[LOOT_ROLL_TYPE_GREED] = "Interface/Buttons/UI-GroupLoot-Coin-Up"
lootIcons[LOOT_ROLL_TYPE_PASS] = "Interface/Buttons/UI-GroupLoot-Pass-Up"
lootIcons[LOOT_ROLL_TYPE_DISENCHANT] = "Interface/Buttons/UI-GroupLoot-DE-Up"

local ICON_PATTERN = "|T%s:0:0:0:0|t"
local function getIconString(choice)
	if not choice then
		debug("no choice")
		return ""
	else
		local icon = lootIcons[choice]
		if not icon then
			debug("no icon found for ", choice)
			return ""
		end -- or 1.0.0-savedvars
		return ICON_PATTERN:format(icon)
	end
end

local expandedRollIDs = {}

local function expanditem(_, rollID)
	if expandedRollIDs[rollID] then
		expandedRollIDs[rollID] = nil
	else
		expandedRollIDs[rollID] = true
	end
	WhoRoll:UpdateTooltip()
end

local tooltip
local function addItemLine(itemLink, shouldExpand)
	if not itemLink then
		local line = tooltip:AddLine("unknown item")
		tooltip:SetCell(line, 2, itemString, nil, "LEFT", 2)
		return line
	end
	local itemName, _, quality, _, _, _, _,_,_, texture = GetItemInfo(itemLink)
	local itemString
	if not itemName or not quality or not texture then
		itemString = itemLink
		texture = "Interface/ICONS/INV_Misc_QuestionMark"
	else
		local _,_,_,hex = GetItemQualityColor(quality)
		itemString = ("|c%s%s|r"):format(hex, itemName or UNKNOWN)
	end
	itemString = ICON_PATTERN:format(texture) .. itemString
	local textureString
	if shouldExpand then
		textureString = ICON_PATTERN:format("Interface\\Buttons\\UI-MinusButton-UP")
	else
		textureString = ICON_PATTERN:format("Interface\\Buttons\\UI-PlusButton-UP")
	end
	local line = tooltip:AddLine(textureString)
	tooltip:SetCell(line, 2, itemString, nil, "LEFT", 2)
	return line
end

local function updateTooltip()
	if not qt:IsAcquired(TOOLTIPNAME) then
		tooltip = qt:Acquire(TOOLTIPNAME, 3)
	else
		tooltip:Clear()
	end
	local title = tooltip:AddHeader()
	tooltip:SetCell(title, 1, "WhoRoll", tooltip:GetHeaderFont(), "LEFT", 3)
	local count = 0
	local active = 0
	for itemIdx = 1, math.min(C_LootHistory.GetNumItems()) do
		if count >= WhoRoll.db.profile.maxHistory then
			break
		end
		local rollID, itemLink, numPlayers, isDone, winnerIdx = C_LootHistory.GetItem(itemIdx);
		if not isDone then
			active = active + 1
		end
		if itemLink then
			local itemName, _, quality, _, _, _, _,_,_, texture = GetItemInfo(itemLink)
			if quality >= WhoRoll.db.profile.qualityThreshold or not isDone then
				local line = addItemLine(itemLink, expandedRollIDs[rollID] or not isDone)
				tooltip:SetLineScript(line, "OnMouseUp", expanditem, rollID)
				if expandedRollIDs[rollID] or not isDone then
					for playerIdx = 1, numPlayers do
						local name, class, rollType, roll, isWinner, isMe = C_LootHistory.GetPlayerInfo(itemIdx, playerIdx);
						tooltip:AddLine(nil, getIconString(rollType), ("%s (%s)"):format(name, roll or ""))
					end
				else
					local name, class, rollType, roll, isWinner, isMe = C_LootHistory.GetPlayerInfo(itemIdx, winnerIdx);
					tooltip:AddLine(nil, getIconString(rollType), ("%s (%s)"):format(name, roll or ""))
				end
				count = count + 1
			end
		end
	end
	tooltip:AddSeparator()
	local foot = tooltip:AddLine()
	tooltip:SetCell(foot, 1, L["|c0000cc00RightClick|r for options"], nil, "LEFT", 3)
	return active
end

local function countCurrentLoots()
	local count = 0
	for itemIdx = 1, C_LootHistory.GetNumItems() do
		local rollID, itemLink, numPlayers, isDone, winnerIdx = C_LootHistory.GetItem(itemIdx);
		if not isDone then
			count = count + 1
		end
	end
	return count
end

local doUpdate = countCurrentLoots
function WhoRoll:UpdateTooltip()
	local count = doUpdate()
	if count > 0 then
		count = "|cff1eff00" .. count .. "|r"
	end
	self.obj.text = string.format(L["%s Rolls"], count)
end

local ldbdata = {
	type = "data source",
	icon = [[Interface/Buttons/UI-GroupLoot-Dice-Up]],
	OnClick = function(_, msg)
			if msg == "LeftButton" then
				updateTooltip()
			elseif msg == "RightButton" then
				WhoRoll:ShowConfig()
			end
		end,
	label = "WhoRoll",
	OnEnter = function (self)
		doUpdate = updateTooltip
		doUpdate()
		tooltip:SmartAnchorTo(self)
		tooltip:SetAutoHideDelay(0.2, self)
		tooltip:Show()
	end,
	OnLeave = function (self)
		-- doUpdate = countCurrentLoots
		-- tooltip:Hide()
	end,
}

WhoRoll.obj = LibStub("LibDataBroker-1.1"):NewDataObject("WhoRoll", ldbdata)
